/*
 * file CqQuery.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cq.CqQuery
 *
 *  Copyright IBM Corporation 2004, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cq;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import java.io.File;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.wvcm.Feedback;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.PropertyRequestItem.PropertyRequest;

import com.ibm.rational.wvcm.stp.StpProvider;
import com.ibm.rational.wvcm.stp.cq.CqFieldValue.ValueType;
import com.ibm.rational.wvcm.stpex.StpExBase;
import com.ibm.rational.wvcm.stpex.StpExEnumeration;
import com.ibm.rational.wvcm.stpex.StpExEnumerationBase;

/**
 * A proxy interface for ClearQuest query and chart resources.
 * <p>
 * The user-friendly specification for the location of a query or chart has the
 * form
 * 
 * <pre>
 *  <b>cq.query:</b><i>&lt;parent-folder-path&gt;</i>/<i>&lt;query-name&gt;</i>@<i>&lt;db-set&gt;</i>/<i>&lt;user-db&gt;</i>
 * </pre>
 * 
 * where <i>&lt;parent-folder-path&gt;</i> is a sequence of one or more path
 * segments separated by the slash character, wherein each segment denotes a
 * query folder immediately contained by its predecessor in the sequence. The
 * first segment is either "Public Queries" or "Personal Queries" (or the
 * localization of one of these).
 */
public interface CqQuery extends CqQueryFolderItem
{
    /**
     * An interface specifying a set of optional parameters that may be passed
     * to
     * {@link CqQuery#doExecute(File, Feedback, com.ibm.rational.wvcm.stp.cq.CqQuery.FileOptions, com.ibm.rational.wvcm.stp.cq.CqQuery.FilterLeaf[]) CqQuery.doExecute},
     * {@link CqQuery#doMakeChart(OutputStream, Feedback, com.ibm.rational.wvcm.stp.cq.CqQuery.ChartOptions, com.ibm.rational.wvcm.stp.cq.CqQuery.FilterLeaf[]) CqQuery.doMakeChart},
     * {@link CqRecordType#doQuery(String, File, Feedback, com.ibm.rational.wvcm.stp.cq.CqQuery.FileOptions) CqRecordType.doQuery},
     * or
     * {@link CqReport#doMakeReport(OutputStream, long, long, com.ibm.rational.wvcm.stp.cq.CqQuery.FilterLeaf[]) CqReport.doMakeReport}
     * to control the execution of queries and the generation of charts and
     * reports.
     * <p>
     * Each method returns a value for one parameter. If that method returns
     * <b>null</b> a system-defined default value will be used as the value for
     * that parameter.
     */
    interface CommonOptions
    {
        /**
         * @return The largest row number that the query may return. This is the
         *         maximum number of rows allowed in the result set <i>only</i>
         *         if the <code>targetRow</code> parameter is 1. In general,
         *         the maximum number of rows returned by an execution of the
         *         query will be the smallest of
         *         <ul>
         *         <li>the <code>maxRows</code> parameter,
         *         <li>the value of
         *         <code>1+getRowNumberLimit()-targetRow</code>, and
         *         <li><code>1+CqResultSet.getRowNumberHardLimit()-targetRows</code>
         *         </ul>. If <b>null</b>, the value of
         *         CqResultSet.getRowNumberSoftLimit() will be used.
         * 
         * @see CqResultSet#getRowNumberHardLimit()
         * @see CqResultSet#getRowNumberSoftLimit()
         * @see CqResultSet#isRowNumberLimitExceeded()
         */
        Long getRowNumberLimit();

        /**
         * @return The properties of the query being executed that should be
         *         retrieved and returned with the result set. If <b>null</b>,
         *         CqResultSet().getQuery will return <b>null</b>; otherwise it
         *         will return a CqQuery proxy populated with the properties
         *         requested.
         * 
         * @see CqResultSet#getQuery()
         */
        PropertyRequest getQueryPropertyRequest();

    }

    /**
     * An extension of the CommonOptions interface specifying an additional set
     * of parameters applying only to the generation of charts.
     */
    interface ChartOptions extends CommonOptions
    {
        /**
         * The possible chart image formats
         */
        public static enum ImageFormat implements StpExEnumeration
        {
            /** Generate the chart in JPEG format */
            JPEG,

            /** Generate the chart in PNG format */
            PNG;
        }

        /**
         * Returns the requested height of the chart image in pixels.
         * 
         * @return The height of the image in pixels.
         */
        Long getHeight();

        /**
         * Returns the compression format requested for the chart image.
         * 
         * @return An ImageFormat enumerator specifying whether the generated
         *         chart image should use JPEG or PNG compression format.
         */
        ImageFormat getImageFormat();

        /**
         * Returns whether or not the compression of the image should be
         * optimized.
         * 
         * @return <b>true</b> if the compression is to be optimized.
         */
        Boolean getIsCompressionOptimized();

        /**
         * Returns whether the chart image is to be rendered in grey scale or
         * color.
         * 
         * @return If <b>true</b>, the chart image will be in grey scale, not
         *         color.
         */
        Boolean getIsGreyScale();

        /**
         * Returns whether or not a PNG image should be interlaced.
         * 
         * @return If <b>true</b>, a PNG image will be interlaced.
         */
        Boolean getIsInterlaced();

        /**
         * Returns whether or not a JPEG image is to use progressive
         * compression.
         * 
         * @return If <b>true</b>, progressive compression will be used in the
         *         JPEG format.
         */
        Boolean getIsProgressive();

        /**
         * Returns the requested quality of JPEG compression, expressed as an
         * integer between 1 (maximum compression) and 100 (maximum fidelity).
         * 
         * @return An Integer between 1 and 100.
         */
        Integer getQuality();

        /**
         * Returns the requested width of the chart image.
         * 
         * @return An Long specifying the requested width of the chart image in
         *         pixels.
         */
        Long getWidth();
    }

    /**
     * Instances of this class provide the default values for each attribute
     * specified by the ChartOptions interface, which is the same behavior
     * exhibited when using <b>null</b> for a ChartOptions object. Clients can
     * provide non-default attribute values by defining and instantiating a
     * subclass of this class that overrides select methods to provide the
     * desired non-default value.
     * <p>
     * Using a class instance creation expression, for example, one could define
     * a ChartOptions object for requesting a row limit of 10000 as follows...
     * 
     * <pre>
     *     ChartOptions TEN_THOUSAND_ROWS = new ChartOptionsClass(){
     *         public Long getRowNumberLimit() { return 10000; };
     *     };
     * </pre>
     */
    public static class ChartOptionsClass implements ChartOptions
    {
        public Long getHeight()
        {
            return null;
        }

        public ImageFormat getImageFormat()
        {
            return null;
        }

        public Boolean getIsCompressionOptimized()
        {
            return null;
        }

        public Boolean getIsGreyScale()
        {
            return null;
        }

        public Boolean getIsInterlaced()
        {
            return null;
        }

        public Boolean getIsProgressive()
        {
            return null;
        }

        public Integer getQuality()
        {
            return null;
        }

        public Long getWidth()
        {
            return null;
        }

        public Long getRowNumberLimit()
        {
            return null;
        }

        public PropertyRequest getQueryPropertyRequest()
        {
            return null;
        }
    }

    /**
     * An instance of ChartOptions that requests the chart in PNG format.
     * Provided as a convenience for specifying this commonly used option.
     */
    CqQuery.ChartOptions PNG_CHART = new CqQuery.ChartOptionsClass()
    {
        public ImageFormat getImageFormat()
        {
            return ImageFormat.PNG;
        }
    };

    /**
     * An extension of the CommonOptions interface specifying an additional set
     * of parameters applying only to the generation of query result sets.
     */
    interface ListOptions extends CommonOptions
    {
        /**
         * If TRUE, the total number of rows generated by the query will be
         * computed by the sever and returned as the value of
         * {@link CqResultSet#getRowCount()}. All of these rows will be in the
         * response only if maxRows is at least as great as the row count. If
         * FALSE, no count is computed; CqResultSet.getRowCount() returns -1. In
         * this latter case, if the number of elements in the response is equal
         * to maxRows, there is no way to tell if there were more rows in the
         * result set not returned by the response.
         * 
         * @return TRUE if doExeute/doQuery is to compute and return a count of
         *         the number of resources that match the filter (whether or not
         *         those resources are actually returned from the server); FALSE
         *         or <b>null</b> if only the maximum number of records are to
         *         be returned with no indication whether or not there might be
         *         more.
         */
        Boolean getEnableRowCount();

        /**
         * @return The maximum number of characters that are to be included in
         *         each row of the response for each multi-line text display
         *         field. If <b>null</b>, the default limit established by the
         *         server will be applied.
         */
        Long getMaxMultiLineTextLength();

        /**
         * As the value returned by getValueConvertionData(), suppresses all
         * data conversions.
         */
        DisplayField[] NO_CONVERSION = new DisplayField[0];

        /**
         * As the value returned by getValueConvertionData(), causes NULL_IMAGE
         * strings to be converted to <b>null</b>s.
         */
        DisplayField[] NULL_CONVERSION = new DisplayField[0];

        /**
         * As the value returned by getValueConvertionData(), causes each
         * NULL_IMAGE string to be converted to <b>null</b> and each other
         * string to be converted to the native Java data type specified for the
         * column in {@link CqResultSet#getColumnTypes()}.
         * <p>
         * Since an SQL-based query has no controlling DisplayField[] array,
         * this conversion is for SQL-based query uses only rudimentary types.
         */
        DisplayField[] FULL_CONVERSION = new DisplayField[0];

        /**
         * Specifies how CqRowData is to interpret the string data for a row
         * when computing the Object[] returned by CqRowData.getValues().
         * <p>
         * If this method returns the distinguished value NO_CONVERSION, the
         * array returned by CqRowData.getValues() will be the same array of
         * Strings returned by CqRowData.getStrings(); otherwise, getValues()
         * will return an array of native Java objects obtained by converting
         * the string image in each column to an object of the type specified
         * for the column in the DisplayField[] returned by this option.
         * (NULL_IMAGE Strings will be converted to <b>null</b>).
         * <p>
         * Enabling this conversion will require some extra overhead in
         * executing the query since the type information must be collected for
         * each column and that may require additional round-trips to the server
         * if not already specified in the DisplayField[] returned by this
         * option. Conversion to Java objects for a particular row is not
         * attempted, however, until the getValues() method is invoked on that
         * row.
         * <p>
         * The default behavior is to use the type information returned by
         * {@link CqResultSet#getColumnTypes()} to perform the conversion when
         * requested.
         * <p>
         * Another possible return is a non-empty DisplayField[], which is
         * useful primarily when the query is SQL based. For a SQL-based query,
         * the client must provide the detailed type information based on it's
         * knowledge of the fields returned by the SQL statement it is using. An
         * explicit DisplayField[] might also be returned to avoid an extra
         * round-trip to the server to obtain the DISPLAY_FIELD information. The
         * DisplayField[] used to define a query or used in the invocation of
         * CqFieldDefinition.doQuery would always be a suitable return value,
         * but lighter weight DisplayField[]s will often do as well.
         * <p>
         * If an explicit DisplayField[] is returned, the number of visible
         * fields in the array must be the same as the number of items returned
         * by CqRowData.getStrings(). Each visible DisplayField must define at
         * least one of its Path, FieldType, Aggregation or Function attributes.
         * The remaining attributes are ignored. If only the Path attribute is
         * specified, the FieldType will be obtained from the last
         * CqFieldDefinition of that Path (requesting that information from the
         * server if necessary).
         * <p>
         * If the Aggregation attribute is COUNT, the other attributes are
         * ignored and the column value is converted to an Integer.
         * <p>
         * If the Function attribute is not <b>null</b> and not NONE, the other
         * attributes are ignored and the column value is converted to a Date
         * object.
         * <p>
         * If the FieldType is RESOURCE or RESOURCE_LIST, the Path attribute
         * must be specified so that the type of the resource referenced by that
         * path can be determined. (The CqFieldDefintion.REFERENCED_RECORD_TYPE
         * property is used for this purpose and will be obtained from the
         * server if not defined by the last proxy in the Path.)
         * 
         * @return NO_CONVERSION, NULL_CONVERSION, FULL_CONVERSION, or a
         *         DisplayField[] specifying a visible field for each value
         *         returned for a row.
         */
        DisplayField[] getValueConversionData();
    }

    /**
     * Instances of this class provide the default values for each attribute
     * specified by the ListOptions interface, which is the same behavior
     * exhibited when using <b>null</b> for a ListOptions object. Clients can
     * provide non-default attribute values by defining and instantiating a
     * subclass of this class that overrides select methods to provide the
     * desired non-default value.
     * <p>
     * Using a class instance creation expression, for example, one could define
     * a ListOptions object for requesting row counts as follows...
     * 
     * <pre>
     *     ListOptions COUNT_ROWS = new ListOptionsClass(){
     *         public Boolean getEnableRowCount() { return Boolean.TRUE; };
     *     };
     * </pre>
     */
    public static class ListOptionsClass implements ListOptions
    {
        public Boolean getEnableRowCount()
        {
            return null;
        };

        public Long getMaxMultiLineTextLength()
        {
            return null;
        };

        public DisplayField[] getValueConversionData()
        {
            return null;
        }

        public Long getRowNumberLimit()
        {
            return null;
        }

        public PropertyRequest getQueryPropertyRequest()
        {
            return null;
        }
    }

    /**
     * An instance of ListOptions that enables the row count option for a query
     * execution. Provided as a convenience for specifying this commonly used
     * option.
     */
    CqQuery.ListOptions COUNT_ROWS = new CqQuery.ListOptionsClass()
    {
        public Boolean getEnableRowCount()
        {
            return Boolean.TRUE;
        };
    };

    /**
     * An interface through which a client may, optionally, specify values for
     * various attributes that control the behavior of executing a query to a
     * stream.
     */
    interface FileOptions
    {
        /**
         * @return The maximum number of characters that are to be included in
         *         each row of the response for each multi-line text display
         *         field.
         */
        Long getMaxMultiLineTextLength();

        /**
         * @return The delimiter to use in those file formats that use a
         *         delimiter.
         */
        String getDelimiter();

        /**
         * @return A String containing additional options for specifying the
         *         form and content of the result set file.
         */
        String getOptions();
    }

    /**
     * Instances of this class provide the default values for each attribute
     * specified by the FileOptions interface, which is the same behavior
     * exhibited when using <b>null</b> for a FileOptions parameter. Clients
     * can provide non-default attribute values by defining and instantiating a
     * subclass of this class that overrides select methods to provide the
     * desired non-default value.
     * <p>
     * Using a class instance creation expression, for example, one could define
     * a FileOptions object for requesting the use of tabs as the file
     * delimiter...
     * 
     * <pre>
     *     FileOptions USE_TABS = new FileOptionsClass(){
     *         public String getDelimiter() { return "\t"; };
     *     };
     * </pre>
     */
    public static class FileOptionsClass implements FileOptions
    {
        public Long getMaxMultiLineTextLength()
        {
            return null;
        };

        public String getDelimiter()
        {
            return null;
        }

        public String getOptions()
        {
            return null;
        }
    }

    /**
     * An enumeration of the possible types of queries.
     */
    public static enum QueryType implements StpExEnumeration
    {
        /** The query generates a list of data, a result set. */
        LIST,

        /** The query generates a formatted report. */
        REPORT,

        /** The query generates a chart. */
        CHART;
    }

    /**
     * The description of a column of the result set generated by a query. The
     * main element of a DisplayField is the field of a record that is displayed
     * in the corresponding column of the result set. Some display fields may
     * not be visible in the display but will be used only to sort and group the
     * result set.
     */
    public interface DisplayField
    {
        /**
         * Supported DisplayField sort options
         */
        public static enum SortType implements StpExEnumeration
        {
            /**
             * The display field does not participate in sorting the result set
             */
            NO_SORT,

            /** The display field values are sorted in ascending order */
            ASCENDING,

            /** The display field values are sorted in descending order */
            DESCENDING;
        }

        /**
         * Supported display field aggregate functions. When these aggregate
         * functions are used, the result set will have only one row. The value
         * in the column of that row will be computed as a function of the
         * values in this display field in all records selected by the query
         * filter.
         */
        public static enum Aggregation implements StpExEnumeration
        {
            /** The column is not aggregated */
            NO_AGGREGATION,

            /**
             * The number of non-<b>null</b> field values associated with this
             * column.
             */
            COUNT,

            /** The sum of the field values associated with his column */
            SUM,

            /** The average of the field values associated with this column */
            AVERAGE,

            /**
             * The minimum value of the field values associated with this column
             */
            MINIMUM,

            /**
             * The maximum value of the field values associated with this column
             */
            MAXIMUM;
        }

        /**
         * The functions that can be applied to a DisplayField of type DATE_TIME
         */
        public static enum Function implements StpExEnumeration
        {
            /** No function has been associated with this field */
            NONE,

            /** The day portion of the field value. For example, 8/29/2002. */
            DAY,

            /**
             * The week number of the year and the year, separated by a comma.
             * For example, 35,2002 for 8/29/2002 (the 35th week of the year).
             */
            WEEK,

            /**
             * The first of the month. For example, 8/1/2002 for the field
             * value, 8/29/2002.
             */
            MONTH,

            /**
             * The date that is the first of the year. For example, 1/1/2002 for
             * the field value, 8/29/2002.
             */
            YEAR;
        }

        /**
         * The value types for result set rows
         */
        public static enum FieldType implements StpExEnumeration
        {
            /** An initial value for ValueType variables */
            UNKNOWN(ValueType.UNKNOWN),

            /**
             * A single-line of text up to 254 characters. Leading and trailing
             * whitespace is trimmed when stored in a record.
             */
            SHORT_STRING(ValueType.SHORT_STRING),

            /**
             * An arbitrarily long string of characters, possibly containing one
             * or more newline characters.
             */
            MULTILINE_STRING(ValueType.MULTILINE_STRING),

            /**
             * An arbitrarily large string of uninterpreted data bits
             */
            BINARY(ValueType.UNKNOWN),

            /** An integer in the range -9999999999 to 9999999999 */
            INTEGER(ValueType.INTEGER),

            /**
             * A GMT date/time between 12:00:01 AM January 1, 1970 and 11:59:59
             * PM January 18, 2038.
             */
            DATE_TIME(ValueType.DATE_TIME),

            /** A floating point value */
            FLOAT(ValueType.FLOAT),

            /** A reference field */
            REFERENCE(ValueType.RESOURCE),

            /** A record id (alphanumeric) field */
            ID(ValueType.ID),

            /** A database id (integer) field */
            DBID(ValueType.DBID),

            /** The name of a record's current state */
            STATE(ValueType.STATE),

            /** The type of a record's current state */
            STATE_TYPE(ValueType.STATE_TYPE),

            /** A record's record type name */
            RECORD_TYPE(ValueType.RECORD_TYPE);

            /**
             * @return The CqFieldValue.ValueType to which this
             *         DisplayField.FieldType equates; ValueType.UNKNOWN if
             *         there is no equivalent ValueType.
             */
            public ValueType valueType()
            {
                return m_valueType;
            }

            private FieldType(ValueType valueType)
            {
                m_valueType = valueType;
            }

            private ValueType m_valueType;
        }

        /**
         * Returns the aggregate function that is applied to the field path of
         * each selected record to compute the value for the result set.
         * 
         * @return An Aggregation type code indicating which, if any, aggregate
         *         function will be used to compute this display field.
         */
        Aggregation getAggregation();

        /**
         * Sets the aggregate function for this display field.
         * 
         * @param aggregation An Aggregation type code. Must not be <b> null</b>.
         */
        void setAggregation(Aggregation aggregation);

        /**
         * Returns the description of the field from the schema.
         * 
         * @return A String containing the description associated with this
         *         field in the schema definition.
         */
        String getDescription();

        /**
         * Returns the FieldType of the display field. This is the data type of
         * the field specified by the Path attribute unless the display field is
         * aggregated, in which case the aggregation function determines this
         * value: <i>e.g.</i>, getFieldType would return INT for aggregates of
         * IDs.
         * 
         * @return The CqFieldValue for this display field.
         */

        FieldType getFieldType();

        /**
         * Returns the simple scalar function that is to be applied to this
         * DisplayField (<i>e.g.</i> month(date))
         * 
         * @return A Function enumerator specifying the scalar function that is
         *         to be applied to the field before inclusion in the result
         *         set.
         */
        Function getFunction();

        /**
         * Sets the simple scalar function that is to be applied to this
         * DisplayField before inclusion in the result set.
         * 
         * @param function The Function enumerator specifying the function.
         */
        void setFunction(Function function);

        /**
         * Returns whether or not the field is in a group-by clause.
         * 
         * @return <b>true</b> if the result set is grouped by this display
         *         field; otherwise <b>false</b>.
         */
        boolean getIsGroupBy();

        /**
         * Sets whether or not this field is in the group-by clause when
         * aggregation functions are used.
         * 
         * @param isGroupBy If <b>true</b>, this field is in the group-by
         *            clause.
         */
        void setIsGroupBy(boolean isGroupBy);

        /**
         * Returns whether or not this field can be used in a query filter.
         * 
         * @return <b>true</b> if and only if this field can be used in a
         *         filtering expression.
         */
        boolean getIsLegalForFilter();

        /**
         * Returns whether or not this display field occupies a column in the
         * result set. Some display fields are used only for sorting, grouping,
         * or filtering and need not be included in the record data returned by
         * the query.
         * 
         * @return <b>true</b> if and only if this display field is included in
         *         the result set.
         */
        boolean getIsVisible();

        /**
         * Sets whether or not this display field occupies a column in the
         * result set
         * 
         * @param isVisible <b>true</b> if this display field is to be included
         *            in the result set; <b>false</b> otherwise.
         */
        void setIsVisible(boolean isVisible);

        /**
         * Returns the label for the column of the result set that contains
         * values from this display field.
         * 
         * @return A character image that may be used as the heading for the
         *         column of the display in which values from this field are
         *         rendered. Will never be <b>null</b>.
         */
        String getLabel();

        /**
         * Sets the label for the column of the result set that contains values
         * from this display field.
         * 
         * @param label A String containing the label for the display field
         *            column. Must not be <b>null</b>.
         */
        void setLabel(String label);

        /**
         * Returns the field path that describes the data source for this
         * display field.
         * 
         * @return A specification of the property from which the value for this
         *         display field is obtained. The path consists of one or more
         *         properties, each represented by a PropertyName object. The
         *         first property in the path is a property of the query's
         *         primary resource type. All properties in the path, except for
         *         the last field, must be resource-valued properties, and in
         *         each case, the property that follows the resource-valued
         *         property must be a property of the resource type referenced
         *         by the preceding resource-valued property. Will never be <b>
         *         null</b>.
         */
        public CqFieldDefinition[] getPath();

        /**
         * Sets the field path that describes the data source for this display
         * field.
         * 
         * @param arg An non-empty array of CqFieldDefinition objects specifying
         *            a property path as described in the documentation for
         *            {@link DisplayField#getPath()}. Must not be <b>null</b>.
         */
        public void setPath(CqFieldDefinition... arg);

        /**
         * Returns the character image of the field path that is the data source
         * for this display field. This value is computed once and cached for
         * future use.
         * 
         * @return A dot-separated list of the names of the properties in the
         *         field path. Will never be <b>null</b>. Will be <b>""</b> if
         *         not yet set.
         */
        public String getPathName();

        /**
         * Returns a code indicating how this field path participates in the
         * sorting of result set records: ascending, descending or not at all.
         * 
         * @return A SortType code indicating how this display field is sorted.
         */
        public SortType getSortType();

        /**
         * Sets the SortType code for this display field.
         * 
         * @param sortType A SortType code. Must not be <b>null</b>.
         */
        public void setSortType(SortType sortType);

        /**
         * Returns the sort order for fields participating in the sort (SortType
         * not equal NO_SORT). The field having sort order 1 is the primary sort
         * key; sort order 2 is the secondary sort key, and so forth.
         * 
         * @return A positive number indicating this display field's ordinal
         *         position within the sort key, or zero if this display field
         *         is not in the sort key.
         */
        public long getSortOrder();

        /**
         * Sets the sort order position for this display field.
         * 
         * @param sortOrder A non-negative integer.
         */
        public void setSortOrder(long sortOrder);

    }

    /**
     * The conjunction or disjunction of subexpressions within a query filter
     * expression. This structure is used to represent a node within the filter
     * expression tree. It represents the logical conjunction (ANDing) or
     * disjunction (ORing) of filter subexpressions.
     */
    public interface FilterNode extends CqQuery.Filter
    {
        /**
         * Returns the n-th operand of the conjunction or disjunction.
         * 
         * @param n The 0-based index of the operand desired. Must not exceed
         *            the number of operands defined for the Operation of this
         *            node.
         * 
         * @return A Filter structure representing the n-th operand of the
         *         subexpression. Will never be <b>null</b>.
         */
        public CqQuery.Filter getOperand(int n);

        /**
         * Returns the first operand of this FilterNode.
         * 
         * @return {@link #getOperand(int) getOperand(0)}
         */
        public CqQuery.Filter getOperand();

        /**
         * Returns the number of operands in this node.
         * 
         * @return The number of operands currently specified in this node.
         */
        public int getOperandCount();

        /**
         * Returns all operands of the conjunction or disjunction represented by
         * this node.
         * 
         * @return A vector of FilterNode structures representing the operands
         *         of the subexpression. Will never be <b>null</b>.
         */
        public CqQuery.Filter[] getOperands();

        /**
         * Sets all operands of the conjunction or disjunction represented by
         * this node.
         * 
         * @param list An array of Filter objects representing the operands of
         *            this FilterNode.
         */
        public void setOperands(CqQuery.Filter... list);
    }

    /**
     * A field/value comparison within a query filter expression; the leaf of a
     * query filter expression. This structure is also used for specifying a
     * dynamic filter value.
     */
    public interface FilterLeaf extends CqQuery.Filter
    {
        /**
         * A code indicating how the target of the comparison is being
         * specified.
         * <p>
         * A TargetType is paired with each target value in a FilterLeaf.
         * <p>
         * For a dynamic filter, the target type is PROMPTED. The target value
         * is the prompt that can be shown to a user to indicate to that user
         * what value that user should provide. Prior to executing the query,
         * the prompt should be replaced by the user-provided value and the
         * target type adjusted appropriately.
         * <p>
         * If the target type is USER, YESTERDAY, TODAY, or TOMMORROW, the
         * target value should be <b>null</b> since the target value is
         * implicit in the target type code.
         * <p>
         * In ClearQuest, filter targets are always expressed as strings. In
         * this API targets may be specified by any Java Object. If the target
         * Object is not a Date, Object.toString() will be applied to it to
         * obtain the string to pass to ClearQuest. If the target value is null,
         * the empty string ("") is used.
         * <p>
         * If the target value is a Date, the Date will be converted to a String
         * formated as expected by ClearQuest, using the value of
         * {@link com.ibm.rational.wvcm.stp.StpProvider#getUserTimeZone()}. The
         * target type with which the Date value is paired determines the format
         * used.
         * <p>
         * If the target value is specified by a String it is not usually
         * modified by the client. But, the value may be reformated by the
         * client if the client knows that the source field or target value is
         * supposed to be a date/time specification or if the UNKNOWN target
         * type is used.
         */
        public static enum TargetType implements StpExEnumeration
        {
            /**
             * Unspecified target type; the <b>null</b> TargetTyoe value. If
             * used in a setTarget call, heuristics will be applied to the
             * associated target value in an attempt to recognize it as a time
             * specification and reformat it to the format expected by
             * ClearQuest if needed.
             */
            UNKNOWN,

            /**
             * Target is a constant value. If the field is of type DATE_TIME,
             * the target type will be converted to DATE_TIME unless the target
             * value is specified as a String and that string contains no time
             * component, in which case the target type will be converted to
             * DATE_ONLY.
             */
            CONSTANT,

            /** Target is a user-supplied value */
            PROMPTED,

            /** Target is the current user */
            USER,

            /** Target is yesterday's date */
            YESTERDAY,

            /** Target is today's date */
            TODAY,

            /** Target is tomorrow's date */
            TOMORROW,
            
            /**
             * Target is the specification for a date-only target. The field
             * value is to be compared against the 24 hour interval for the
             * specified date. If a Date object is used to specify the target
             * value, only the date portion of that object will be used in the
             * filter, where the date is determined using the current client
             * time zone defined in the provider. If a String is used to specify
             * the target value and that specification has a time component,
             * the string will be parsed to a Date using the client time zone
             * and then formatted as a date-only string as just described.
             */
            DATE_ONLY,

            /**
             * Target is the specification for a date and time target. The field
             * value is to be compared against the precise time denoted by the
             * target value. If a Date object is used to specify the target
             * value its value will be used in the filter. If a String is used
             * to specify the target value and the String contains no time
             * component, the value will be converted to midnight on the date
             * specified in the client time zone.
             */
            DATE_TIME,
        }

        /**
         * Returns the field path designating the source of the comparison.
         * 
         * @return A specification of the record field whose value will be
         *         compared against the target value(s) in this filter. The path
         *         consists of one or more record fields each represented by a
         *         {@link CqFieldDefinition CqFieldDefinition} object. The first
         *         field in the path is a field of the query's primary record
         *         type. All fields in the path, except for the last field, must
         *         be reference-type fields, and in each case, the field that
         *         follows the reference field must be a field of the record
         *         type referenced by the preceding reference field. Will never
         *         be <b>null</b>.
         */
        public CqFieldDefinition[] getSource();

        /**
         * Returns the character image of the field path designating the source
         * of the comparison
         * 
         * @return The returned image consists of the field names of the field
         *         path separated one from the other by a dot symbol. Will never
         *         be <b>null</b>.
         */
        public String getSourceName();

        /**
         * Returns the number of targets in this comparison expression Each
         * comparison operation requires a different number of targets ranging
         * from none (IS_NULL, IS_NOT_NULL) to many (IS_IN_SET, IS_NOT_IN_SET).
         * 
         * @return The number of comparison targets currently specified for this
         *         comparison.
         */
        public int getTargetCount();

        /**
         * Returns the target type code for the n-th target of the comparison.
         * 
         * @param index The 0-based index of the desired target. Must be less
         *            than the target count.
         * 
         * @return The target type code for the n-th target of the comparison.
         */
        public FilterLeaf.TargetType getTargetType(int index);

        /**
         * Returns the target type code for the first target.
         * 
         * @return getTargetType(0)
         */
        public FilterLeaf.TargetType getTargetType();

        /**
         * Returns the n-th target value of the comparison.
         * 
         * @param index The 0-based index of the desired target. Must be less
         *            than the target count.
         * 
         * @return The character image of the the n-th target. Will never be
         *         <b>null</b>.
         */
        public String getTarget(int index);

        /**
         * Returns the first target of the comparison.
         * 
         * @return {@link #getTarget(int) getTarget(0)}.
         */
        public String getTarget();

        /**
         * Sets the first target of the comparison and resets the target count
         * to 1.
         * 
         * @param targettype
         *            The type of the first target value. Cannot be <b> null</b>.
         * @param target
         *            The first target value. The value provided will be
         *            converted to a String object appropriate to the TargetType
         *            specified. If null and a string value is required, the
         *            empty string ("") will be used.
         */
        public void setTarget(FilterLeaf.TargetType targettype,
                              Object target);

        /**
         * Sets the first target of the comparison and resets the target count
         * to 1.
         * 
         * @param targetType A FilterLeaf.TargetType enumerator specifying the
         *            type of the first target. This TargetType must be
         *            TODAY, TOMORROW, YESTERDAY, or USER
         */
        public void setTarget(FilterLeaf.TargetType targetType);

        /**
         * Add the type and value for an additional target of the comparison.
         * 
         * @param targetType The type of the additional target value. Cannot be
         *            <b> null</b>.
         * @param target The additional target. The value provided will be
         *            converted to a String object appropriate to the TargetType
         *            specified. If null and a string value is required, the
         *            empty string ("") will be used.
         */
        public void addTarget(FilterLeaf.TargetType targetType,
                              Object target);

        /**
         * Returns a list of all the target values for the comparison. If the
         * TargetType of the target is TODAY, TOMORROW, YESTERDAY, or USER, the
         * value is <b>null</b>
         * 
         * @return A list of String elements, each containing the character
         *         image of a target value. Will never be <b>null</b>.
         */
        public List<String> getTargets();

        /**
         * Returns an array of all the target types for the comparison
         * corresponding to the vector of values returned by {@link
         * CqQuery.FilterLeaf#getTargets() FilterLeaf.getTargets()}.
         * 
         * @return A vector of TargetType codes corresponding in order to the
         *         values returned by {@link CqQuery.FilterLeaf#getTargets()
         *         FilterLeaf.getTargets()}.
         */
        public TargetType[] getTargetTypes();

        /**
         * Set the types and values of multiple targets for the comparison.
         * 
         * @param targetTypes
         *            An array of target type codes designating the way the
         *            corresponding values in the targets parameter are
         *            specified.
         * @param targets
         *            An array of Object elements, each containing a target
         *            value for the comparison. Cannot be <b>null</b>. The
         *            values provided will be converted to a String objects
         *            appropriate to the TargetTypes specified. If <b>null</b>
         *            and a string value is required, the empty string ("") will
         *            be used.
         */
        void setTargets(TargetType[] targetTypes,
                        List<Object> targets);

        /**
         * Set the type and value for each targets for the comparison.
         * 
         * @param targets
         *            An array of TargetType codes and other Object values
         *            designating the new targets of the comparison operation.
         *            <p>
         *            Nominally, this argument is an array of alternating
         *            TargetType and target value Objects specifying,
         *            respectively, the type and value of each filter target.
         *            However, for USER, YESTERDAY, TODAY, and TOMORROW target
         *            types, the associated <b>null</b> value may be elided. If
         *            a target value is not preceded by a target type, then the
         *            target type is assumed to be CONSTANT or UNKNOWN. Thus,
         *            although a TargetType may always be specified for each
         *            target value, a TargetType entry is is <i>required</i>
         *            only when specifying a PROMPTED, DATE_ONLY, or DATE_TIME
         *            target type.
         *            <p>
         *            Initially, an untyped value is assumed to be of type
         *            UNKNOWN, which means heuristics will be applied to the
         *            value to determine whether or not the value is a date/time
         *            value that needs to be converted to a form acceptable to
         *            ClearQuest. If, however, the CONSTANT type was used
         *            previously in the array, an untyped value is assumed to be
         *            of type CONSTANT, which means the value will be converted
         *            only if the field type is known to be DATE_TIME.
         *            <p>
         *            May be <b>null</b> or empty to specify no targets for the
         *            comparison operation.
         */
        void setTargets(Object... targets);
    }

    /**
     * A query filter expression or a subexpression thereof.
     * 
     * <p>
     * This structure represents a node within the filter expression tree. As a
     * FilterNode, it represents the logical conjunction (ANDing) or disjunction
     * (ORing) of filter subexpressions. As a FilterLeaf, it represents the
     * comparison of a field with one or more constant values.
     */

    public interface Filter
    {
        /**
         * An enumeration of the logical operations and comparisons that can be
         * used to combine the operands of a Filter subexpression.
         */
        public static enum Operation implements StpExEnumeration
        {
            /** A FilterNode with an unspecified operation */
            UNSPECIFIED,

            /** A FilterNode representing the logical AND of its operands */
            CONJUNCTION,

            /** A FilterNode representing the logical OR of its operands */
            DISJUNCTION,

            /** A FilterLeaf with an unspecified comparison */
            UNSPECIFIED_COMPARISON,

            /** test the source and target for equality */
            IS_EQUAL,

            /** test the source and target for inequality */
            IS_NOT_EQUAL,

            /** test that the source is strictly less than the target */
            IS_LESS_THAN,

            /** test that the source is less than or equal to the target */
            IS_LESS_THAN_OR_EQUAL,

            /** test that the source is strictly greater than the target */
            IS_GREATER_THAN,

            /** test that the source is greater than or equal to the target */
            IS_GREATER_THAN_OR_EQUAL,

            /** test that the target is a substring of the source */
            HAS_SUBSTRING,

            /** test that the target is not a substring of the source */
            HAS_NO_SUBSTRING,

            /** test that the source is between the two target values */
            IS_BETWEEN,

            /** test that the source is not between the two targets */
            IS_NOT_BETWEEN,

            /** test that the source is <b>null</b> */
            IS_NULL,

            /** test that the source is not <b>null</b> */
            IS_NOT_NULL,

            /** test that the source is equal to one of the targets */
            IS_IN_SET,

            /** test that the source is not equal to any of the targets */
            IS_NOT_IN_SET;
        }

        /**
         * Returns the logical operation that is to be used to combine the
         * operands of this subexpression. You can use the operation code to
         * determine what kind of subexpression this structure represents before
         * casting it to a FilterNode or FilterLeaf. Operations UNSPECIFIED,
         * CONJUNCTION and DISJUNCTION indicate a FilterNode; the remaining
         * Operations indicate a FilterLeaf.
         * 
         * @return The Operation enumerator for the logical or comparison
         *         operator of this node.
         */
        public Operation getOperation();

        /**
         * Set the operation code for a Filter subexpression.
         * 
         * @param op The new operation code. The code must be consistent with
         *            the most complete subclass of this object.
         */
        public void setOperation(Operation op);
    }

    /**
     * A special field path token designating the aggregated key fields of a
     * ClearQuest record, for use in specifying query filters and display
     * fields.
     */
    CqFieldDefinition[] UNIQUE_KEY_FIELD_PATH = new CqFieldDefinition[0];

    /**
     * Returns the position/column number of the display field having the given
     * label. Requires that the DISPLAY_FIELDS property is defined for this
     * proxy.
     * 
     * @param label A String containing the column label for the desired display
     *            field.
     * 
     * @return 0 if the label cannot be found amongst the visible display fields
     *         of the query; otherwise the ordinal position of the display field
     *         among the columns of the result set.
     * 
     * @throws WvcmException If the DISPLAY_FIELDS property is not defined for
     *             this proxy.
     */
    long columnNumber(String label) throws WvcmException;

    /**
     * Creates a new query at the location specified by this proxy and delivers
     * it to the database. This proxy must define the PRIMARY_RECORD_TYPE
     * property and one of the properties DISPLAY_FIELDS or SQL.
     * 
     * @param feedback Requests the properties desired in the proxy returned by
     *            this operation.
     * @return A proxy for the newly created query populated with the properties
     *         requested by the feedback argument.
     * @throws WvcmException If a resource already exists at the location
     *             defined by this proxy or if the new query cannot be delivered
     *             to the database for any number of reasons.
     */
    CqQuery doCreateQuery(Feedback feedback) throws WvcmException;

    /**
     * Creates a new query at the location specified by this proxy and delivers
     * it to the database. This proxy must always define the PRIMARY_RECORD_TYPE
     * property. At least one of the properties DISPLAY_FIELDS or SQL must be
     * defined if the new query is to be delivered to the database.
     * 
     * @param feedback Requests the properties desired in the proxy returned by
     *            this operation.
     * @param deliveryOrder A deliver order list specifying the resources to be
     *            delivered to the database after this resource is successfully
     *            created.
     * @return A proxy for the newly created query populated with the properties
     *         requested by the feedback argument.
     * @throws WvcmException If a resource already exists at the location
     *             defined by this proxy or if the resources specified by the
     *             deliveryOrder argument cannot be delivered to the database.
     */
    CqQuery doCreateQuery(Feedback feedback,
                          List<CqContextResource> deliveryOrder)
        throws WvcmException;

    /**
     * Executes this query on the server and returns rows of the result set as
     * requested.
     * 
     * @param targetRow The 1-based index of a row of the result set that must
     *            be included in the response to this request. The target row
     *            values will be the values returned in the first RowData object
     *            of the iterator.
     * @param maxRows The maximum number of rows of the result set that should
     *            be included in the response to this request. A value of
     *            Long.MAX_VALUE will request all rows in the result set; If
     *            this parameter is 0, no rows are requested. If the query does
     *            not generate all of the rows requested, it is not an error.
     * @param options An instance of ListOptions that specifies optional
     *            attributes for controlling the performance of the query or its
     *            response. May be <b>null</b>, in which case server-specified
     *            values will be used for each attribute.
     * @param dynamicFilters Some queries require selected portions of the
     *            filter to be specified dynamically at the time the query is
     *            executed. These filter values must be specified in the
     *            dynamicFilters vector in the order defined by the query. See
     *            {@link CqQuery#DYNAMIC_FILTERS Query#DYNAMIC_FILTERS}. May be
     *            <b>null</b> if the query has no parameters.
     * 
     * @return A CqResultSet presenting the rows of the result set in the order
     *         returned by the query.
     * 
     * @throws WvcmException if any errors occur during the execution of the
     *             query or the formation of the response.
     */
    CqResultSet doExecute(long targetRow,
                          long maxRows,
                          ListOptions options,
                          FilterLeaf... dynamicFilters) throws WvcmException;

    /**
     * Executes this query and writes the result set to a text file
     * 
     * @param results The File into which the result set is to be written. Must
     *            not be <b>null</b>.
     * @param feedback A Feedback instance specifying properties to be returned
     *            from the operation in the CqQuery proxy. May be <b>null</b>.
     * @param options A StreamOptions instance specifying various options to
     *            control the format of the generated output.
     * @param dynamicFilters Some queries require selected portions of the
     *            filter to be specified dynamically at the time the query is
     *            executed. These filter values must be specified in the
     *            dynamicFilters vector in the order defined by the query. See
     *            {@link CqQuery#DYNAMIC_FILTERS Query#DYNAMIC_FILTERS}. May be
     *            <b>null</b> if the query has no parameters.
     * @return A CqQuery proxy for the query that was executed, populated with
     *         the property values requested by the feedback parameter.
     * @throws WvcmException
     */
    CqQuery doExecute(File results,
                      Feedback feedback,
                      FileOptions options,
                      FilterLeaf... dynamicFilters) throws WvcmException;

    /**
     * Executes this query and formats the results into a chart.
     * 
     * @param chartImage An OutputStream to which the generated chart image will
     *            be written.
     * @param feedback A Feedback instance specifying properties to be returned
     *            from the operation in the CqQuery proxy. May be <b>null</b>.
     * @param options A ChartOptions object specifying the attributes of the
     *            generated chart.
     * @param dynamicFilters Any dynamic filters needed to execute the query.
     * @return A proxy for this query populated with the properties specified by
     *         the Feedback argument.
     * @throws WvcmException Thrown if the chart cannot be generated as
     *             requested.
     */
    CqQuery doMakeChart(OutputStream chartImage,
                        Feedback feedback,
                        ChartOptions options,
                        FilterLeaf... dynamicFilters) throws WvcmException;

    /**
     * An array containing a {@link CqQuery.FilterLeaf  FilterLeaf} structure
     * for each of the query's dynamic filters. This property is computed from
     * the PROMPTED leaves of the filter defined by the FILTERING property.
     */
    PropertyName<CqQuery.FilterLeaf[]> DYNAMIC_FILTERS =
        new PropertyName<CqQuery.FilterLeaf[]>(PROPERTY_NAMESPACE,
                                               "dynamic-filters");

    /**
     * Returns the value of the {@link CqQuery#DYNAMIC_FILTERS DYNAMIC_FILTERS}
     * property as defined by this proxy.
     * 
     * @return An array of FilterLeaf objects, each representing one of the
     *         dynamic filters defined for this query. An empty array if the
     *         query's filtering expression has no PROMPTED FieldLeaf instances.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link CqQuery#DYNAMIC_FILTERS DYNAMIC_FILTERS} property.
     */
    CqQuery.FilterLeaf[] getDynamicFilters() throws WvcmException;

    /**
     * Whether or not any fields of the query are aggregated.
     */
    PropertyName<Boolean> IS_AGGREGATED =
        new PropertyName<Boolean>(StpExBase.PROPERTY_NAMESPACE, "is-aggregated");

    /**
     * Returns the value of the {@link CqQuery#IS_AGGREGATED IS_AGGREGATED}
     * property as defined by this proxy.
     * 
     * @return <b>true</b> if this query contains any aggregated display
     *         fields; <b>false</b> otherwise.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link CqQuery#IS_AGGREGATED IS_AGGREGATED} property.
     */
    boolean getIsAggregated() throws WvcmException;

    /**
     * Whether or not the query is defined on a family of record types as
     * opposed to being defined on a single record type.
     */
    PropertyName<Boolean> IS_MULTITYPE =
        new PropertyName<Boolean>(StpExBase.PROPERTY_NAMESPACE, "is-multitype");

    /**
     * Returns the value of the {@link CqQuery#IS_MULTITYPE IS_MULTITYPE}
     * property as defined by this proxy.
     * 
     * @return <b>true</b> if this query is defined on a family of record
     *         types; <b>false</b> if it is defined on a single record type.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link CqQuery#IS_MULTITYPE IS_MULTITYPE} property.
     */
    boolean getIsMultitype() throws WvcmException;

    /**
     * Whether or not SQL has been generated for the query
     */
    PropertyName<Boolean> IS_SQL_GENERATED =
        new PropertyName<Boolean>(StpExBase.PROPERTY_NAMESPACE,
                                  "is-sql-generated");

    /**
     * Returns the value of the
     * {@link CqQuery#IS_SQL_GENERATED IS_SQL_GENERATED} property as defined by
     * this proxy.
     * 
     * @return <b>true</b> if SQL has been generated for this query; <b>false</b>
     *         if the SQL has been provided by the user.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link CqQuery#IS_SQL_GENERATED IS_SQL_GENERATED} property.
     */
    boolean getIsSqlGenerated() throws WvcmException;

    /**
     * The record type of the records that are targeted by the query.
     */
    PropertyName<CqRecordType> PRIMARY_RECORD_TYPE =
        new PropertyName<CqRecordType>(PROPERTY_NAMESPACE,
                                       "primary-record-type");

    /**
     * Returns the value of the {@link CqQuery#PRIMARY_RECORD_TYPE
     * PRIMARY_RECORD_TYPE} property as defined by this proxy.
     * 
     * @return A CqRecordType proxy for the primary resource type of this query.
     *         Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link CqQuery#PRIMARY_RECORD_TYPE PRIMARY_RECORD_TYPE}
     *             property.
     */
    CqRecordType getPrimaryRecordType() throws WvcmException;

    /**
     * Defines a new value for the {@link CqQuery#PRIMARY_RECORD_TYPE
     * PRIMARY_RECORD_TYPE} property of this proxy.
     * 
     * @param primaryRecordType A CqRecordType object representing the new
     *            primary record type for this query definition.
     */
    void setPrimaryRecordType(CqRecordType primaryRecordType);

    /**
     * Whether this query generates a list, report, or chart
     */
    PropertyName<CqQuery.QueryType> QUERY_TYPE =
        new PropertyName<CqQuery.QueryType>(StpExBase.PROPERTY_NAMESPACE,
                                            "query-type");

    /**
     * Returns the value of the {@link CqQuery#QUERY_TYPE QUERY_TYPE} property
     * as defined by this proxy.
     * 
     * @return A QueryType enumerator identifying the type of this query.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link CqQuery#QUERY_TYPE QUERY_TYPE} property.
     */
    CqQuery.QueryType getQueryType() throws WvcmException;

    /**
     * The SQL associated with the query.
     */
    PropertyName<String> SQL =
        new PropertyName<String>(StpExBase.PROPERTY_NAMESPACE, "sql");

    /**
     * Returns the value of the {@link CqQuery#SQL SQL} property as defined by
     * this proxy.
     * 
     * @return A String containing the SQL statement for this query.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link CqQuery#SQL SQL} property.
     */
    String getSql() throws WvcmException;

    /**
     * Defines a new value for the {@link CqQuery#SQL SQL} property of this
     * proxy.
     * 
     * @param sql A String object containing the SQL statement for the query.
     */
    void setSql(String sql);

    /**
     * An array containing a {@link DisplayField  DisplayField} structure for
     * each column of the result set generated by this query.
     */
    PropertyName<DisplayField[]> DISPLAY_FIELDS =
        new PropertyName<DisplayField[]>(StpExBase.PROPERTY_NAMESPACE,
                                         "display-fields");

    /**
     * Returns the value of the {@link CqQuery#DISPLAY_FIELDS DISPLAY_FIELDS}
     * property as defined by this proxy.
     * 
     * @return An array of DisplayField structures describing the columns of the
     *         result set generated by the query.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link CqQuery#DISPLAY_FIELDS DISPLAY_FIELDS} property.
     */
    DisplayField[] getDisplayFields() throws WvcmException;

    /**
     * Defines a new value for the {@link CqQuery#DISPLAY_FIELDS DISPLAY_FIELDS}
     * property of this proxy.
     * 
     * @param displayFields An array of DisplayField objects representing the
     *            new display fields of the query.
     */
    void setDisplayFields(DisplayField... displayFields);

    /**
     * A {@link CqQuery.Filter  Filter} node representing the logical expression
     * that is used to filter records to form each row of the result set.
     */
    PropertyName<CqQuery.Filter> FILTERING =
        new PropertyName<CqQuery.Filter>(StpExBase.PROPERTY_NAMESPACE,
                                         "filtering");

    /**
     * Returns the value of the {@link CqQuery#FILTERING FILTERING} property as
     * defined by this proxy.
     * 
     * @return A Filter object representing the filter expression defined by
     *         this query definition. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link CqQuery#FILTERING FILTERING} property.
     */
    CqQuery.Filter getFiltering() throws WvcmException;

    /**
     * Defines a new value for the {@link CqQuery#FILTERING FILTERING} property
     * of this proxy.
     * 
     * @param filter A Filter object representing the new filtering to be used
     *            in this query definition.
     */
    void setFiltering(CqQuery.Filter filter);
}
